/* autogenerated by ./tidy_linktypes_list.sh */
#ifdef DLT_NULL
LT(DLT_NULL)
#endif
#ifdef DLT_EN10MB
LT(DLT_EN10MB)
#endif
#ifdef DLT_EN3MB
LT(DLT_EN3MB)
#endif
#ifdef DLT_AX25
LT(DLT_AX25)
#endif
#ifdef DLT_PRONET
LT(DLT_PRONET)
#endif
#ifdef DLT_CHAOS
LT(DLT_CHAOS)
#endif
#ifdef DLT_IEEE802
LT(DLT_IEEE802)
#endif
#ifdef DLT_ARCNET
LT(DLT_ARCNET)
#endif
#ifdef DLT_SLIP
LT(DLT_SLIP)
#endif
#ifdef DLT_PPP
LT(DLT_PPP)
#endif
#ifdef DLT_FDDI
LT(DLT_FDDI)
#endif
#ifdef DLT_ATM_RFC1483
LT(DLT_ATM_RFC1483)
#endif
#ifdef DLT_RAW
LT(DLT_RAW)
#endif
#ifdef DLT_RAW
LT(DLT_RAW)
#endif
#ifdef DLT_SLIP_BSDOS
LT(DLT_SLIP_BSDOS)
#endif
#ifdef DLT_PPP_BSDOS
LT(DLT_PPP_BSDOS)
#endif
#ifdef DLT_SLIP_BSDOS
LT(DLT_SLIP_BSDOS)
#endif
#ifdef DLT_PPP_BSDOS
LT(DLT_PPP_BSDOS)
#endif
#ifdef DLT_PFSYNC
LT(DLT_PFSYNC)
#endif
#ifdef DLT_ATM_CLIP
LT(DLT_ATM_CLIP)
#endif
#ifdef DLT_REDBACK_SMARTEDGE
LT(DLT_REDBACK_SMARTEDGE)
#endif
#ifdef DLT_PPP_SERIAL
LT(DLT_PPP_SERIAL)
#endif
#ifdef DLT_PPP_ETHER
LT(DLT_PPP_ETHER)
#endif
#ifdef DLT_SYMANTEC_FIREWALL
LT(DLT_SYMANTEC_FIREWALL)
#endif
#ifdef DLT_MATCHING_MIN
LT(DLT_MATCHING_MIN)
#endif
#ifdef DLT_C_HDLC
LT(DLT_C_HDLC)
#endif
#ifdef DLT_CHDLC
LT(DLT_CHDLC)
#endif
#ifdef DLT_IEEE802_11
LT(DLT_IEEE802_11)
#endif
#ifdef DLT_FRELAY
LT(DLT_FRELAY)
#endif
#ifdef DLT_LOOP
LT(DLT_LOOP)
#endif
#ifdef DLT_LOOP
LT(DLT_LOOP)
#endif
#ifdef DLT_ENC
LT(DLT_ENC)
#endif
#ifdef DLT_ENC
LT(DLT_ENC)
#endif
#ifdef DLT_LINUX_SLL
LT(DLT_LINUX_SLL)
#endif
#ifdef DLT_LTALK
LT(DLT_LTALK)
#endif
#ifdef DLT_ECONET
LT(DLT_ECONET)
#endif
#ifdef DLT_IPFILTER
LT(DLT_IPFILTER)
#endif
#ifdef DLT_PFLOG
LT(DLT_PFLOG)
#endif
#ifdef DLT_CISCO_IOS
LT(DLT_CISCO_IOS)
#endif
#ifdef DLT_PRISM_HEADER
LT(DLT_PRISM_HEADER)
#endif
#ifdef DLT_AIRONET_HEADER
LT(DLT_AIRONET_HEADER)
#endif
#ifdef DLT_PFSYNC
LT(DLT_PFSYNC)
#endif
#ifdef DLT_HHDLC
LT(DLT_HHDLC)
#endif
#ifdef DLT_IP_OVER_FC
LT(DLT_IP_OVER_FC)
#endif
#ifdef DLT_SUNATM
LT(DLT_SUNATM)
#endif
#ifdef DLT_RIO
LT(DLT_RIO)
#endif
#ifdef DLT_PCI_EXP
LT(DLT_PCI_EXP)
#endif
#ifdef DLT_AURORA
LT(DLT_AURORA)
#endif
#ifdef DLT_IEEE802_11_RADIO
LT(DLT_IEEE802_11_RADIO)
#endif
#ifdef DLT_TZSP
LT(DLT_TZSP)
#endif
#ifdef DLT_ARCNET_LINUX
LT(DLT_ARCNET_LINUX)
#endif
#ifdef DLT_JUNIPER_MLPPP
LT(DLT_JUNIPER_MLPPP)
#endif
#ifdef DLT_JUNIPER_MLFR
LT(DLT_JUNIPER_MLFR)
#endif
#ifdef DLT_JUNIPER_ES
LT(DLT_JUNIPER_ES)
#endif
#ifdef DLT_JUNIPER_GGSN
LT(DLT_JUNIPER_GGSN)
#endif
#ifdef DLT_JUNIPER_MFR
LT(DLT_JUNIPER_MFR)
#endif
#ifdef DLT_JUNIPER_ATM2
LT(DLT_JUNIPER_ATM2)
#endif
#ifdef DLT_JUNIPER_SERVICES
LT(DLT_JUNIPER_SERVICES)
#endif
#ifdef DLT_JUNIPER_ATM1
LT(DLT_JUNIPER_ATM1)
#endif
#ifdef DLT_APPLE_IP_OVER_IEEE1394
LT(DLT_APPLE_IP_OVER_IEEE1394)
#endif
#ifdef DLT_MTP2_WITH_PHDR
LT(DLT_MTP2_WITH_PHDR)
#endif
#ifdef DLT_MTP2
LT(DLT_MTP2)
#endif
#ifdef DLT_MTP3
LT(DLT_MTP3)
#endif
#ifdef DLT_SCCP
LT(DLT_SCCP)
#endif
#ifdef DLT_DOCSIS
LT(DLT_DOCSIS)
#endif
#ifdef DLT_LINUX_IRDA
LT(DLT_LINUX_IRDA)
#endif
#ifdef DLT_IBM_SP
LT(DLT_IBM_SP)
#endif
#ifdef DLT_IBM_SN
LT(DLT_IBM_SN)
#endif
#ifdef DLT_USER0
LT(DLT_USER0)
#endif
#ifdef DLT_USER1
LT(DLT_USER1)
#endif
#ifdef DLT_USER2
LT(DLT_USER2)
#endif
#ifdef DLT_USER3
LT(DLT_USER3)
#endif
#ifdef DLT_USER4
LT(DLT_USER4)
#endif
#ifdef DLT_USER5
LT(DLT_USER5)
#endif
#ifdef DLT_USER6
LT(DLT_USER6)
#endif
#ifdef DLT_USER7
LT(DLT_USER7)
#endif
#ifdef DLT_USER8
LT(DLT_USER8)
#endif
#ifdef DLT_USER9
LT(DLT_USER9)
#endif
#ifdef DLT_USER10
LT(DLT_USER10)
#endif
#ifdef DLT_USER11
LT(DLT_USER11)
#endif
#ifdef DLT_USER12
LT(DLT_USER12)
#endif
#ifdef DLT_USER13
LT(DLT_USER13)
#endif
#ifdef DLT_USER14
LT(DLT_USER14)
#endif
#ifdef DLT_USER15
LT(DLT_USER15)
#endif
#ifdef DLT_IEEE802_11_RADIO_AVS
LT(DLT_IEEE802_11_RADIO_AVS)
#endif
#ifdef DLT_JUNIPER_MONITOR
LT(DLT_JUNIPER_MONITOR)
#endif
#ifdef DLT_BACNET_MS_TP
LT(DLT_BACNET_MS_TP)
#endif
#ifdef DLT_PPP_PPPD
LT(DLT_PPP_PPPD)
#endif
#ifdef DLT_PPP_WITH_DIRECTION
LT(DLT_PPP_WITH_DIRECTION)
#endif
#ifdef DLT_LINUX_PPP_WITHDIRECTION
LT(DLT_LINUX_PPP_WITHDIRECTION)
#endif
#ifdef DLT_JUNIPER_PPPOE
LT(DLT_JUNIPER_PPPOE)
#endif
#ifdef DLT_JUNIPER_PPPOE_ATM
LT(DLT_JUNIPER_PPPOE_ATM)
#endif
#ifdef DLT_GPRS_LLC
LT(DLT_GPRS_LLC)
#endif
#ifdef DLT_GPF_T
LT(DLT_GPF_T)
#endif
#ifdef DLT_GPF_F
LT(DLT_GPF_F)
#endif
#ifdef DLT_GCOM_T1E1
LT(DLT_GCOM_T1E1)
#endif
#ifdef DLT_GCOM_SERIAL
LT(DLT_GCOM_SERIAL)
#endif
#ifdef DLT_JUNIPER_PIC_PEER
LT(DLT_JUNIPER_PIC_PEER)
#endif
#ifdef DLT_ERF_ETH
LT(DLT_ERF_ETH)
#endif
#ifdef DLT_ERF_POS
LT(DLT_ERF_POS)
#endif
#ifdef DLT_LINUX_LAPD
LT(DLT_LINUX_LAPD)
#endif
#ifdef DLT_JUNIPER_ETHER
LT(DLT_JUNIPER_ETHER)
#endif
#ifdef DLT_JUNIPER_PPP
LT(DLT_JUNIPER_PPP)
#endif
#ifdef DLT_JUNIPER_FRELAY
LT(DLT_JUNIPER_FRELAY)
#endif
#ifdef DLT_JUNIPER_CHDLC
LT(DLT_JUNIPER_CHDLC)
#endif
#ifdef DLT_MFR
LT(DLT_MFR)
#endif
#ifdef DLT_JUNIPER_VP
LT(DLT_JUNIPER_VP)
#endif
#ifdef DLT_A429
LT(DLT_A429)
#endif
#ifdef DLT_A653_ICM
LT(DLT_A653_ICM)
#endif
#ifdef DLT_USB_FREEBSD
LT(DLT_USB_FREEBSD)
#endif
#ifdef DLT_USB
LT(DLT_USB)
#endif
#ifdef DLT_BLUETOOTH_HCI_H4
LT(DLT_BLUETOOTH_HCI_H4)
#endif
#ifdef DLT_IEEE802_16_MAC_CPS
LT(DLT_IEEE802_16_MAC_CPS)
#endif
#ifdef DLT_USB_LINUX
LT(DLT_USB_LINUX)
#endif
#ifdef DLT_CAN20B
LT(DLT_CAN20B)
#endif
#ifdef DLT_IEEE802_15_4_LINUX
LT(DLT_IEEE802_15_4_LINUX)
#endif
#ifdef DLT_PPI
LT(DLT_PPI)
#endif
#ifdef DLT_IEEE802_16_MAC_CPS_RADIO
LT(DLT_IEEE802_16_MAC_CPS_RADIO)
#endif
#ifdef DLT_JUNIPER_ISM
LT(DLT_JUNIPER_ISM)
#endif
#ifdef DLT_IEEE802_15_4_WITHFCS
LT(DLT_IEEE802_15_4_WITHFCS)
#endif
#ifdef DLT_IEEE802_15_4
LT(DLT_IEEE802_15_4)
#endif
#ifdef DLT_SITA
LT(DLT_SITA)
#endif
#ifdef DLT_ERF
LT(DLT_ERF)
#endif
#ifdef DLT_RAIF1
LT(DLT_RAIF1)
#endif
#ifdef DLT_IPMB_KONTRON
LT(DLT_IPMB_KONTRON)
#endif
#ifdef DLT_JUNIPER_ST
LT(DLT_JUNIPER_ST)
#endif
#ifdef DLT_BLUETOOTH_HCI_H4_WITH_PHDR
LT(DLT_BLUETOOTH_HCI_H4_WITH_PHDR)
#endif
#ifdef DLT_AX25_KISS
LT(DLT_AX25_KISS)
#endif
#ifdef DLT_LAPD
LT(DLT_LAPD)
#endif
#ifdef DLT_PPP_WITH_DIR
LT(DLT_PPP_WITH_DIR)
#endif
#ifdef DLT_C_HDLC_WITH_DIR
LT(DLT_C_HDLC_WITH_DIR)
#endif
#ifdef DLT_FRELAY_WITH_DIR
LT(DLT_FRELAY_WITH_DIR)
#endif
#ifdef DLT_LAPB_WITH_DIR
LT(DLT_LAPB_WITH_DIR)
#endif
#ifdef DLT_IPMB_LINUX
LT(DLT_IPMB_LINUX)
#endif
#ifdef DLT_FLEXRAY
LT(DLT_FLEXRAY)
#endif
#ifdef DLT_MOST
LT(DLT_MOST)
#endif
#ifdef DLT_LIN
LT(DLT_LIN)
#endif
#ifdef DLT_X2E_SERIAL
LT(DLT_X2E_SERIAL)
#endif
#ifdef DLT_X2E_XORAYA
LT(DLT_X2E_XORAYA)
#endif
#ifdef DLT_IEEE802_15_4_NONASK_PHY
LT(DLT_IEEE802_15_4_NONASK_PHY)
#endif
#ifdef DLT_LINUX_EVDEV
LT(DLT_LINUX_EVDEV)
#endif
#ifdef DLT_GSMTAP_UM
LT(DLT_GSMTAP_UM)
#endif
#ifdef DLT_GSMTAP_ABIS
LT(DLT_GSMTAP_ABIS)
#endif
#ifdef DLT_MPLS
LT(DLT_MPLS)
#endif
#ifdef DLT_USB_LINUX_MMAPPED
LT(DLT_USB_LINUX_MMAPPED)
#endif
#ifdef DLT_DECT
LT(DLT_DECT)
#endif
#ifdef DLT_AOS
LT(DLT_AOS)
#endif
#ifdef DLT_WIHART
LT(DLT_WIHART)
#endif
#ifdef DLT_FC_2
LT(DLT_FC_2)
#endif
#ifdef DLT_FC_2_WITH_FRAME_DELIMS
LT(DLT_FC_2_WITH_FRAME_DELIMS)
#endif
#ifdef DLT_IPNET
LT(DLT_IPNET)
#endif
#ifdef DLT_CAN_SOCKETCAN
LT(DLT_CAN_SOCKETCAN)
#endif
#ifdef DLT_IPV4
LT(DLT_IPV4)
#endif
#ifdef DLT_IPV6
LT(DLT_IPV6)
#endif
#ifdef DLT_IEEE802_15_4_NOFCS
LT(DLT_IEEE802_15_4_NOFCS)
#endif
#ifdef DLT_DBUS
LT(DLT_DBUS)
#endif
#ifdef DLT_JUNIPER_VS
LT(DLT_JUNIPER_VS)
#endif
#ifdef DLT_JUNIPER_SRX_E2E
LT(DLT_JUNIPER_SRX_E2E)
#endif
#ifdef DLT_JUNIPER_FIBRECHANNEL
LT(DLT_JUNIPER_FIBRECHANNEL)
#endif
#ifdef DLT_DVB_CI
LT(DLT_DVB_CI)
#endif
#ifdef DLT_MUX27010
LT(DLT_MUX27010)
#endif
#ifdef DLT_STANAG_5066_D_PDU
LT(DLT_STANAG_5066_D_PDU)
#endif
#ifdef DLT_JUNIPER_ATM_CEMIC
LT(DLT_JUNIPER_ATM_CEMIC)
#endif
#ifdef DLT_NFLOG
LT(DLT_NFLOG)
#endif
#ifdef DLT_NETANALYZER
LT(DLT_NETANALYZER)
#endif
#ifdef DLT_NETANALYZER_TRANSPARENT
LT(DLT_NETANALYZER_TRANSPARENT)
#endif
#ifdef DLT_IPOIB
LT(DLT_IPOIB)
#endif
#ifdef DLT_MPEG_2_TS
LT(DLT_MPEG_2_TS)
#endif
#ifdef DLT_NG40
LT(DLT_NG40)
#endif
#ifdef DLT_NFC_LLCP
LT(DLT_NFC_LLCP)
#endif
#ifdef DLT_PFSYNC
LT(DLT_PFSYNC)
#endif
#ifdef DLT_INFINIBAND
LT(DLT_INFINIBAND)
#endif
#ifdef DLT_SCTP
LT(DLT_SCTP)
#endif
#ifdef DLT_USBPCAP
LT(DLT_USBPCAP)
#endif
#ifdef DLT_RTAC_SERIAL
LT(DLT_RTAC_SERIAL)
#endif
#ifdef DLT_BLUETOOTH_LE_LL
LT(DLT_BLUETOOTH_LE_LL)
#endif
#ifdef DLT_WIRESHARK_UPPER_PDU
LT(DLT_WIRESHARK_UPPER_PDU)
#endif
#ifdef DLT_NETLINK
LT(DLT_NETLINK)
#endif
#ifdef DLT_BLUETOOTH_LINUX_MONITOR
LT(DLT_BLUETOOTH_LINUX_MONITOR)
#endif
#ifdef DLT_BLUETOOTH_BREDR_BB
LT(DLT_BLUETOOTH_BREDR_BB)
#endif
#ifdef DLT_BLUETOOTH_LE_LL_WITH_PHDR
LT(DLT_BLUETOOTH_LE_LL_WITH_PHDR)
#endif
#ifdef DLT_PROFIBUS_DL
LT(DLT_PROFIBUS_DL)
#endif
#ifdef DLT_PKTAP
LT(DLT_PKTAP)
#endif
#ifdef DLT_PKTAP
LT(DLT_PKTAP)
#endif
#ifdef DLT_EPON
LT(DLT_EPON)
#endif
#ifdef DLT_IPMI_HPM_2
LT(DLT_IPMI_HPM_2)
#endif
#ifdef DLT_ZWAVE_R1_R2
LT(DLT_ZWAVE_R1_R2)
#endif
#ifdef DLT_ZWAVE_R3
LT(DLT_ZWAVE_R3)
#endif
#ifdef DLT_WATTSTOPPER_DLM
LT(DLT_WATTSTOPPER_DLM)
#endif
#ifdef DLT_ISO_14443
LT(DLT_ISO_14443)
#endif
#ifdef DLT_RDS
LT(DLT_RDS)
#endif
#ifdef DLT_USB_DARWIN
LT(DLT_USB_DARWIN)
#endif
#ifdef DLT_OPENFLOW
LT(DLT_OPENFLOW)
#endif
#ifdef DLT_SDLC
LT(DLT_SDLC)
#endif
#ifdef DLT_TI_LLN_SNIFFER
LT(DLT_TI_LLN_SNIFFER)
#endif
#ifdef DLT_LORATAP
LT(DLT_LORATAP)
#endif
#ifdef DLT_VSOCK
LT(DLT_VSOCK)
#endif
#ifdef DLT_NORDIC_BLE
LT(DLT_NORDIC_BLE)
#endif
#ifdef DLT_DOCSIS31_XRA31
LT(DLT_DOCSIS31_XRA31)
#endif
#ifdef DLT_ETHERNET_MPACKET
LT(DLT_ETHERNET_MPACKET)
#endif
#ifdef DLT_DISPLAYPORT_AUX
LT(DLT_DISPLAYPORT_AUX)
#endif
#ifdef DLT_LINUX_SLL2
LT(DLT_LINUX_SLL2)
#endif
#ifdef DLT_SERCOS_MONITOR
LT(DLT_SERCOS_MONITOR)
#endif
#ifdef DLT_OPENVIZSLA
LT(DLT_OPENVIZSLA)
#endif
#ifdef DLT_EBHSCR
LT(DLT_EBHSCR)
#endif
#ifdef DLT_VPP_DISPATCH
LT(DLT_VPP_DISPATCH)
#endif
#ifdef DLT_DSA_TAG_BRCM
LT(DLT_DSA_TAG_BRCM)
#endif
#ifdef DLT_DSA_TAG_BRCM_PREPEND
LT(DLT_DSA_TAG_BRCM_PREPEND)
#endif
#ifdef DLT_IEEE802_15_4_TAP
LT(DLT_IEEE802_15_4_TAP)
#endif
#ifdef DLT_DSA_TAG_DSA
LT(DLT_DSA_TAG_DSA)
#endif
#ifdef DLT_DSA_TAG_EDSA
LT(DLT_DSA_TAG_EDSA)
#endif
#ifdef DLT_ELEE
LT(DLT_ELEE)
#endif
#ifdef DLT_Z_WAVE_SERIAL
LT(DLT_Z_WAVE_SERIAL)
#endif
#ifdef DLT_USB_2_0
LT(DLT_USB_2_0)
#endif
#ifdef DLT_ATSC_ALP
LT(DLT_ATSC_ALP)
#endif
#ifdef DLT_MATCHING_MAX
LT(DLT_MATCHING_MAX)
#endif
